#!/usr/bin/python3
# -*- coding: utf-8 -*-

# Noms : KINADINOVA
# Prenoms : Dariya
# Groupe : MI15
# Date : 24/01/24



"""

:mod: module `recursivite`
:author: FIL - Faculté des Sciences et Technologies - Univ. Lille
:link: <http://portail.fil.univ-lille1.fr>_
:date: Mars 2020
:dernière révision: janvier 2024

"""

from ap_decorators import count, trace
# l'instruction suivante permet d'annoter des paramètres qui sont des functions.
from collections.abc import Callable


def taille_binaire(naturel: int) -> int:
    """
    Renvoie le nombre de chiffres dans l'écriture binaire de l'entier naturel `naturel`

    Précondition :
       naturel >= 0

    Exemples :
    $$$ taille_binaire(0)
    1
    $$$ taille_binaire(1)
    1
    $$$ taille_binaire(2)
    2
    $$$ taille_binaire(1023)
    10
    $$$ taille_binaire(1024)
    11
    $$$ from random import randrange
    $$$ l = [randrange(1,2**100)  for _ in range(100)]
    $$$ all(taille_binaire(elt) == len(bin(elt))-2  for elt in l)
    True

    """
    res = 1
    while naturel >= 2:
        res += 1
        naturel //= 2
    return res

def taille_binaire_recursive(naturel: int) -> int:
    """
    Renvoie le nombre de chiffres dans l'écriture binaire de l'entier naturel `naturel`

    Précondition :
       naturel >= 0

    Exemples :
    $$$ taille_binaire_recursive(0)
    1
    $$$ taille_binaire_recursive(1)
    1
    $$$ taille_binaire_recursive(2)
    2
    $$$ taille_binaire_recursive(1023)
    10
    $$$ taille_binaire_recursive(1024)
    11
    $$$ from random import randrange
    $$$ l = [randrange(1,2**100)  for _ in range(100)]
    $$$ all(taille_binaire_recursive(elt) == len(bin(elt))-2  for elt in l)
    True
    """
    res = 0
    if naturel == 0 or naturel == 1:
        res = 1
    else:
        res = 1 + taille_binaire_recursive(naturel//2)
    return res

def poids_binaire(naturel: int) -> int:
    """
    Renvoie le nombre de chiffre 1 dans l'écriture binaire de l'entier naturel `naturel`

    Précondition :
       naturel >= 0

    Exxemples :

    $$$ poids_binaire(0)
    0
    $$$ poids_binaire(1)
    1
    $$$ poids_binaire(2)
    1
    $$$ poids_binaire(255)
    8
    $$$ from random import randrange
    $$$ l = [randrange(1,2**100)  for _ in range(100)]
    $$$ all([poids_binaire(x)==bin(x).count('1') for x in l])
    True
    """
    res = naturel % 2
    while naturel > 0:
        naturel //= 2
        res += naturel % 2
    return res

def poids_binaire_recursif(naturel: int) -> int:
    """
    Renvoie le nombre de chiffre 1 dans l'écriture binaire de l'entier naturel `naturel`

    Précondition :
       naturel >= 0

    Exxemples :

    $$$ poids_binaire_recursif(0)
    0
    $$$ poids_binaire_recursif(1)
    1
    $$$ poids_binaire_recursif(2)
    1
    $$$ poids_binaire_recursif(255)
    8
    $$$ from random import randrange
    $$$ l = [randrange(1, 2**100)  for _ in range(100)]
    $$$ all([poids_binaire_recursif(x)==bin(x).count('1') for x in l])
    True
    """
    if naturel == 0:
        res = 0
    else:
        res = naturel%2 + (poids_binaire_recursif(naturel//2))
    return res

def puissance(x: int|float, n: int) -> int|float:
    """
    Calcule x élevé à la puissance n

    Précondition :
        n>=0

    Exemples :

    $$$ puissance(10, 0)
    1
    $$$ puissance(10, 1)
    10
    $$$ puissance(2, 10)
    1024
    $$$ puissance(2, 2)
    4
    """
    if n == 0:
        res = 1
    else:
        res = x * puissance(x, n-1)
    return res
        
def puissance_v2(x: int|float, n: int) -> int|float:
    """
    calcule  x élevé à la puissance n

    Précondition :   n>=0

    Exemples :
    $$$ puissance_v2(10,0)
    1
    $$$ puissance_v2(10,1)
    10
    $$$ puissance_v2(2,10)
    1024
    """
    if n == 0:
        res = 1
    else:
        res = fois(x, puissance(x, n-1))
    return res

@count
def fois(x: int|float, y: int|float) -> int|float:
    """
    renvoie le produit de x par y

    Précondition : les mêmes que l'opérateur *

    Exemples :
    $$$ fois(8, 7)
    56
    """
    return x * y

def comptage(puissance: Callable[[int|float, int], int|float]) -> list[int]:
    """
    Renvoie une liste de longueur 100 contenant le nombre de multiplications
    effectuées par la fonction ``puissance`` passée en paramètre

    Précondition :
       la fonction doit être implantée en utilisant la fonction ``fois``
    """
    res = []
    for i in range(100):
        fois.counter = 0
        _ = puissance(2, i)
        res.append(fois.counter)
    return res

# Fucntion comptage takes  the parameter of function puissance and sends 
# a list of number of multiplications done by function fois in the function.

# When calculating comptage(puissance) we get a list of 0s because 
# the function puissance does not use function fois.

# When calculating comptage(puissance_v2) we get a list with a single 0 and the rest being 1.
# That's because puissance_v2 does use function fois, the first time it's not
# used because n == 0.

#

#@trace
def puissance_calbuth(x: int|float, n: int) -> int|float:
    """
    calcule  x élevé à la puissance n

    Précondition :
        n>=0

    Exemples :

    $$$ puissance_calbuth(10,0)
    1
    $$$ puissance_calbuth(10,1)
    10
    $$$ puissance_calbuth(2,10)
    1024

    """
    if n == 0:
        return 1
    if n == 1:
        return x
    else:
        k = n // 2
        return puissance_calbuth(x, k) * puissance_calbuth(x, n - k)

def puissance_calbuth_v2(x: int|float, n: int) -> int|float:
    """
    calcule  x élevé à la puissance n

    Précondition :
        n>=0

    Exemples :

    $$$ puissance_calbuth_v2(10,0)
    1
    $$$ puissance_calbuth_v2(10,1)
    10
    $$$ puissance_calbuth_v2(2,10)
    1024

    """
    if n == 0:
        return 1
    if n == 1:
        return x
    else:
        k = n // 2
        return fois(puissance_calbuth(x, k), puissance_calbuth(x, n - k))

# Yes, it does reduce the complexity. It divides the calculation into
# two which reduces the need for more calculations.

# The reduction of calculations are helpful in the case of
# big numbers as it divides the calculation in two smaller parts.

# There will be more stacked recrusive functions, which may be unconvinient.

# puissance_calbuth(2,2) is calculated 3 times

def puissance_calbuth_v2_amelioree(x: int|float, n: int) -> int|float:
    """
    calcule  x élevé à la puissance n

    Précondition :
        n>=0

    Exemples :

    $$$ puissance_calbuth_v2_amelioree(10,0)
    1
    $$$ puissance_calbuth_v2_amelioree(10,1)
    10
    $$$ puissance_calbuth_v2_amelioree(2,10)
    1024

    """
    if n == 0:
        return 1
    if n == 1:
        return x
    else:
        k = n // 2
        res = puissance_calbuth_v2_amelioree(x, k)
        if n%2 == 0:
            return res * res
        else:
            return res * res * x

def puissance_erronee(x: int|float, n: int) -> int|float:
    """
    aurait dû calculer  x élevé à la puissance n

    Précondition :
       n >= 0

    Exemples :

    $$$ puissance_erronee(10, 0)
    1
    $$$ puissance_erronee(10, 1)
    10
    $$$ #$$$ puissance_erronee(2, 10)
    $$$ #1024
    """
    if n == 0:
        return 1
    elif n == 1:
        return x
    else:
        r = n % 2
        q = n // 2
        return puissance_erronee(x, r) * puissance_erronee(puissance_erronee(x, q), 2)

def puissance_reparee(x: int|float, n: int) -> int|float:
    """
    calcule  x élevé à la puissance n

    Précondition :
        n>=0

    Exemples :

    $$$ puissance_reparee(10,0)
    1
    $$$ puissance_reparee(10,1)
    10
    $$$ puissance_reparee(2,10)
    1024
    """
    ...

